;;; cmucl ffi bindings for librrd
;;; Miles Egan <miles@caddr.com>
;;; $Id: rrd.lisp,v 1.8 2002/06/04 05:52:08 miles Exp $

(in-package :com.caddr.rrd)

;; error objects and helper macros
(define-condition rrderror (error))

(defmacro call-rrd-func (func &rest args)
  (let ((res (gensym)))
    `(progn
       (setf optind 0 opterr 0)
       (rrd_clear_error)
       (let ((,res (funcall ,func ,@args)))
	 (unless (= (rrd_test_error) 0)
	   (signal 'rrderror))
	 ,res))))

(defmacro def-std-rrdfunc (func cfunc)
  `(progn
     (def-alien-routine ,cfunc int (argcount int) (args (* (* char))))
     (declaim (inline ,cfunc))
     (setf (symbol-function ',func)
	   #'(lambda (&rest args)
	       (with-alien ((strarr (* c-string) (build-arg-array args)))
			   (call-rrd-func ',cfunc (1+ (length args)) strarr))))))

;; librrd variables and routines
(def-alien-variable "opterr" int)
(def-alien-variable "optind" int)
(def-alien-routine rrd_clear_error void)
(def-std-rrdfunc create rrd_create)
(def-std-rrdfunc dump rrd_dump)
(def-alien-routine rrd_get_error c-string)
(def-alien-routine rrd_graph int
  (argcount int)
  (args (* (* char)))
  (printout (* (* (* char))))
  (width int :out)
  (height int :out))
(def-std-rrdfunc rlast rrd_last)
(def-std-rrdfunc resize rrd_resize)
(def-std-rrdfunc restore rrd_restore)
(def-alien-routine rrd_test_error int)
(def-std-rrdfunc tune rrd_tune)
(def-std-rrdfunc update rrd_update)

;; glue code
(defun build-arg-array (args)
  (let* ((numargs (1+ (length args)))
	 (strarr (make-alien (c-string) numargs))) 
    (push "dummy" args) ; rrd needs a dummy first argument
    (dotimes (i (length args))
      (setf (deref strarr i) (nth i args)))
    strarr))

(defun graph (&rest args)
  (with-alien ((strarr (* c-string) (build-arg-array args))
	       (strout (* c-string))
	       (width int)
	       (height int))
	      (call-rrd-func #'rrd_graph (1+ (length args)) strarr (addr strout))))
